//+------------------------------------------------------------------+
//|                                          Swef_System_EA_ver2.mq4 |
//|                                                         Zen_Leow |
//|                                                                  |
//+------------------------------------------------------------------+
/*
1. All trades are based on the high or the low of the candle that has just closed. I use GBPUSD but would like to be able to trade any pair.
2. I use the daily chart but if possible I would like the option to test other time periods.
3. orders are placed the minute a new candle starts after checking for matching coditions.
4. If MACD is above zero on yesterday's candle then we only consider longs
5. If MACD is below zero on yesterday's candle then we only consider shorts.
6. If MACD is above zero then we place a long at the lowest price of the candle that has just closed.
7. If MACD is below zero then we place a short at the highest price of the candle that has just closed.
8. Each individual trade that we open has a take profit of 100 pips.
9. No stop losses are attached to individual trades.
10. Each day at about 4pm London time I turn on my computer if the equity level is higher than the amount written down then I close all open and pending trades.
11. If I have six pending trades then I cancel the oldest pending trade. So the maximum number of pending trades is five.
12. If MACD is above zero then all pending sell trades are cancelled.
13. If MCD is below zero then all pending buys are cancelled.
14. Each day at 4pm London time I turn on my computer, if my equity my opened positions are in profit then I close all open trades and all pending trades.
15. If at any time the equity level is 500 pips more or less than the initial equity level then I close all open trades.

What's new in version 2
=======================
1. separated the EquityOut option to choose whether or not to take losses during drawdowns (no equity out on loss by default).
2. allow opening of trades at 45 pips away from the close even when the candle's High and/or Low isn't at least 45 pips away from the close.
*/

#property copyright "Zen_Leow"
#property link      ""
#include <stdlib.mqh>
#include <stderror.mqh>

extern int       EA_MAGIC_NUM = 191243;
extern int       OpeningHour = 0;  // time the orders are placed 
extern int       OrderCheckingHour = 15;  // time the orders are checked
extern bool      ChartHasSundayBar = true;
extern int       MinPipsFromLastClose = 45;
extern bool      CheckMonthlyMACD = false;
extern int       MaxPendingOrders = 5;
extern int       PendingOrderExpirationDays = 5;
extern bool      UseEquityLossOut = true;
extern int       EquityProfitOutPips = 500;
extern int       EquityLossOutPips = 500;

extern int       Slippage = 3;
extern int       TakeProfit = 100;
extern int       StopLoss = 0;

extern bool      MoneyManagement = true;
extern double    RiskPercent = 1.5;
extern double    Lots = 1.0;
extern int       LotsDecimalAllowed = 2;
extern double    MaxLots = 15.0;
extern double    MinLots = 0.01;

string msg = "";
int yesterdayIndex = 1;
//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
{
//----
   bool StartupStatusOK = true;
   if (MinLots < MarketInfo(Symbol(),MODE_MINLOT))
   {
      MinLots = MarketInfo(Symbol(),MODE_MINLOT);
      Alert("MinLots adjusted to: "+MinLots);
      StartupStatusOK = false;
   }
   if (MaxLots > MarketInfo(Symbol(),MODE_MAXLOT))
   {
      MaxLots = MarketInfo(Symbol(),MODE_MAXLOT);
      Alert("MaxLots adjusted to: "+MaxLots);
      StartupStatusOK = false;
   }
   if (OpeningHour < 0 || OpeningHour > 23)
   {
      OpeningHour = 0;
      Alert("Invalid OpeningHour given, adjusted to: "+OpeningHour);
      StartupStatusOK = false;
   }
   if (OrderCheckingHour < 0 || OrderCheckingHour > 23)
   {
      OrderCheckingHour = 15;
      Alert("Invalid OrderCheckingHour given, adjusted to: "+OrderCheckingHour);
      StartupStatusOK = false;
   }
   if (UseEquityLossOut && EquityLossOutPips < 1)
   {
      EquityLossOutPips = 500;
      Alert("Invalid EquityOutPips given, adjusted to: "+EquityLossOutPips);
      StartupStatusOK = false;
   }
   if (MoneyManagement && RiskPercent <= 0)
   {
      RiskPercent = 1.5;
      Alert("Invalid RiskPercent given, adjusted to: "+RiskPercent);
      StartupStatusOK = false;
   }
   if (StartupStatusOK)
   {
      Alert("EA initiated with valid inputs");
   }
   WriteComment();
//----
   return(0);
}
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
{
//----
   
//----
   return(0);
}

void WriteComment()
{
   msg = "";
   if (UseEquityLossOut)
   {
      msg = msg + "Using Equity Loss Out Feature";
   }
   if (MoneyManagement)
   {
      msg = msg + "\nUsing MoneyManagement Feature";
   }
   int CurrentHour = TimeHour(TimeCurrent());
   if (CurrentHour == OpeningHour)
   {
      msg = msg + "\nTime to check MACD for opening trades";
   }
   if (CurrentHour == OrderCheckingHour)
   {
      msg = msg + "\nTime to check currently opened trades";
   }
   if (MACDSaysBuy())
   {
      msg = msg + "\nMACD telling us to open buy";
   }
   if (MACDSaysSell())
   {
      msg = msg + "\nMACD telling us to open sell";
   }
   Comment(msg);
}

double GetDailyMACD(int index)
{
   return (iMACD(NULL,0,8,17,9,PRICE_CLOSE,MODE_MAIN,index));
}

double GetMonthlyMACD(int index)
{
   return (iMACD(NULL,PERIOD_MN1,8,17,9,PRICE_CLOSE,MODE_MAIN,index));
}

bool MACDSaysBuy()
{
   yesterdayIndex = 1;
   if (ChartHasSundayBar)
   {
      if (TimeDayOfWeek(TimeCurrent()) == 1)
      {
         yesterdayIndex = 2; // use friday's candle
      }
      if (TimeDayOfWeek(TimeCurrent()) == 0)
      {
         return (false); // don't place trades on Sundays.
      }
   }
   if (GetDailyMACD(yesterdayIndex) > 0)
   {
      if (CheckMonthlyMACD)
      {
         if (GetMonthlyMACD(0) > 0)
         {
            return (true);
         }
      }
      else
      {
         return (true);
      }
   } 
   return (false);
}

bool MACDSaysSell()
{
   yesterdayIndex = 1;
   if (ChartHasSundayBar)
   {
      if (TimeDayOfWeek(TimeCurrent()) == 1)
      {
         yesterdayIndex = 2; // use friday's candle
      }
      if (TimeDayOfWeek(TimeCurrent()) == 0)
      {
         return (false); // don't place trades on Sundays.
      }
   }
   if (GetDailyMACD(yesterdayIndex) < 0)
   {
      if (CheckMonthlyMACD)
      {
         if (GetMonthlyMACD(0) < 0)
         {
            return (true);
         }
      }
      else
      {
         return (true);
      }
   } 
   return (false);
}

double GetBuyPrice()
{
   yesterdayIndex = 1;
   if (ChartHasSundayBar)
   {
      if (TimeDayOfWeek(TimeCurrent()) == 1)
      {
         yesterdayIndex = 2; // use friday's candle
      }
   }
   if (((Close[yesterdayIndex] - Low[yesterdayIndex])/Point) >= MinPipsFromLastClose)
   {
      return (Low[yesterdayIndex]);
   }
   else
   {
      return (NormalizeDouble((Close[yesterdayIndex] - (MinPipsFromLastClose * Point)),Digits));
   }
}

double GetSellPrice()
{
   yesterdayIndex = 1;
   if (ChartHasSundayBar)
   {
      if (TimeDayOfWeek(TimeCurrent()) == 1)
      {
         yesterdayIndex = 2; // use friday's candle
      }
   }
   if (((High[yesterdayIndex] - Close[yesterdayIndex])/Point) >= MinPipsFromLastClose)
   {
      return (High[yesterdayIndex]);
   }
   else
   {
      return (NormalizeDouble((Close[yesterdayIndex] + (MinPipsFromLastClose * Point)),Digits));
   }
}

bool DecideToOpenTrade(int tradeType, int tradeType2)
{
   int total = OrdersTotal();
   if (total > 0)
   {
      for(int cnt=0;cnt<total;cnt++)
      {
         if(OrderSelect(cnt,SELECT_BY_POS))
         {
            if(OrderSymbol()==Symbol() && OrderMagicNumber() == EA_MAGIC_NUM && ((OrderType()==tradeType) || (OrderType()==tradeType2)))
            {
               if (Time[0] <= OrderOpenTime()) // don't open a new position if we're still on the same candle
               {
                  return (false);
               }
            }
         }
      }
   }
   // in case trades has already opened and closed within the candle
   int histotal = OrdersHistoryTotal();
   if (histotal > 0)
   {
      for(cnt=0;cnt<histotal;cnt++)
      {
         if(OrderSelect(cnt,SELECT_BY_POS,MODE_HISTORY))
         {
            if(OrderSymbol()==Symbol() && OrderMagicNumber() == EA_MAGIC_NUM && ((OrderType()==tradeType) || (OrderType()==tradeType2)))
            {
               if (Time[0] <= OrderOpenTime()) // don't open a new position if we're still on the same candle
               {
                  return (false);
               }
            }
         }
      }
   }
   return (true);
}

double PositionSizeToOpen()
{
   double PositionSize = 0;
   
   if (MoneyManagement)
   {
      PositionSize = ((AccountEquity()/100) * RiskPercent) / (MarketInfo(Symbol(),MODE_LOTSIZE) / AccountLeverage());
   }
   else // use fixed lots
   {
      PositionSize = Lots;
   }
   if (PositionSize < MinLots)
   {
      PositionSize = MinLots;
   }
   if (PositionSize > MaxLots)
   {
      PositionSize = MaxLots;
   }
   PositionSize = NormalizeDouble(PositionSize,LotsDecimalAllowed);
   return (PositionSize);
}

void SendOrders (int BuyOrSell, double LotSize, double PriceToOpen, double Slippage, double SL_Price, double TP_Price, string comments, datetime ExpirationTime)
{
   int ticket, errorType;
   
   if (BuyOrSell == OP_BUYLIMIT)
   {  
      Print("Bid: "+Bid+" Ask: "+Ask+" | Opening Buy Order: "+Symbol()+", "+BuyOrSell+", "+LotSize+", "+PriceToOpen+", "+Slippage+", "+SL_Price+", "+TP_Price+", "+comments+", "+EA_MAGIC_NUM+", "+ExpirationTime+", Green");
      ticket=OrderSend(Symbol(),BuyOrSell,LotSize,PriceToOpen,Slippage,SL_Price,TP_Price,comments,EA_MAGIC_NUM,ExpirationTime,Green);
      if(ticket>0)
      {
         if(OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES)) 
         {
            Print("BUY order opened : ",OrderOpenPrice());
            msg = ticket + ": Buy position opened on "+Symbol()+" at "+ Day()+"/"+Month()+"/"+Year()+" - "+Hour()+":"+Minute()+":"+Seconds();
            WriteToLogFile(msg);
         }
      }
      else 
      {  
         errorType = GetLastError();
         Print("Error opening BUY order : ", ErrorDescription(errorType));
         msg = "CANNOT open BUY position on "+Symbol()+" at "+ Day()+"/"+Month()+"/"+Year()+" - "+Hour()+":"+Minute()+":"+Seconds();
         WriteToLogFile(msg);
      }
   }
   if (BuyOrSell == OP_SELLLIMIT)
   {  
      Print("Bid: "+Bid+" Ask: "+Ask+" | Opening Sell Order: "+Symbol()+", "+BuyOrSell+", "+LotSize+", "+PriceToOpen+", "+Slippage+", "+SL_Price+", "+TP_Price+", "+comments+", "+EA_MAGIC_NUM+", "+ExpirationTime+", Red");
      ticket=OrderSend(Symbol(),BuyOrSell,LotSize,PriceToOpen,Slippage,SL_Price,TP_Price,comments,EA_MAGIC_NUM,ExpirationTime,Red);
      if(ticket>0)
      {
         if(OrderSelect(ticket,SELECT_BY_TICKET,MODE_TRADES)) 
         {
            Print("Sell order opened : ",OrderOpenPrice());
            msg = ticket + ": Sell position opened on "+Symbol()+" at "+ Day()+"/"+Month()+"/"+Year()+" - "+Hour()+":"+Minute()+":"+Seconds();
            WriteToLogFile(msg);
         }
      }
      else 
      {  
         errorType = GetLastError();
         Print("Error opening SELL order : ", ErrorDescription(errorType));
         msg = "CANNOT open SELL position on "+Symbol()+" at "+ Day()+"/"+Month()+"/"+Year()+" - "+Hour()+":"+Minute()+":"+Seconds();
         WriteToLogFile(msg);
      }
   }
}

bool EquityProfitOutReached()
{
   int total = OrdersTotal();
   int totalPips = 0;
   if (total > 0)
   {
      for(int cnt=0;cnt<total;cnt++)
      {
         if(OrderSelect(cnt,SELECT_BY_POS))
         {
            if(OrderSymbol()==Symbol() && OrderMagicNumber() == EA_MAGIC_NUM)
            {
               if (OrderType() == OP_BUY)
               {
                  totalPips = totalPips + ((Bid - OrderOpenPrice()) / Point);
               }
               if (OrderType() == OP_SELL)
               {
                  totalPips = totalPips + ((OrderOpenPrice() - Ask) / Point);
               }
            }
         }
      }
      if (totalPips >= EquityProfitOutPips)
      {
         return (true);
      }
   }
   return (false);
}

bool EquityLossOutReached()
{
   int total = OrdersTotal();
   int totalPips = 0;
   if (total > 0)
   {
      for(int cnt=0;cnt<total;cnt++)
      {
         if(OrderSelect(cnt,SELECT_BY_POS))
         {
            if(OrderSymbol()==Symbol() && OrderMagicNumber() == EA_MAGIC_NUM)
            {
               if (OrderType() == OP_BUY)
               {
                  totalPips = totalPips + ((Bid - OrderOpenPrice()) / Point);
               }
               if (OrderType() == OP_SELL)
               {
                  totalPips = totalPips + ((OrderOpenPrice() - Ask) / Point);
               }
            }
         }
      }
      if (totalPips <= -EquityLossOutPips)
      {
         return (true);
      }
   }
   return (false);
}

bool PositionsAreInProfit()
{
   int total = OrdersTotal();
   double totalProfit = 0;
   if (total > 0)
   {
      for(int cnt=0;cnt<total;cnt++)
      {
         if(OrderSelect(cnt,SELECT_BY_POS))
         {
            if(OrderSymbol()==Symbol() && OrderMagicNumber() == EA_MAGIC_NUM)
            {
               if (OrderType() == OP_BUY || OrderType() == OP_SELL)
               {
                  totalProfit = totalProfit + OrderProfit();
               }
            }
         }
      }
      if (totalProfit > 0)
      {
         return (true);
      }
   }
   return (false);
}

void CloseAllPendingSellTrades()
{
   int total = OrdersTotal()-1;
   int positionType;
   if (total > 0)
   {
      for(int cnt=total;cnt>=0;cnt--)
      {
         if(OrderSelect(cnt,SELECT_BY_POS))
         {
            if(OrderSymbol()==Symbol() && OrderMagicNumber() == EA_MAGIC_NUM)
            {
               positionType = OrderType();
               if (positionType == OP_SELLLIMIT)
               {
                  if (!OrderDelete(OrderTicket()))
                  {
                     OrderError();
                  }
               }
            }
         }
      }
   }
}

void CloseAllPendingBuyTrades()
{
   int total = OrdersTotal()-1;
   int positionType;
   if (total > 0)
   {
      for(int cnt=total;cnt>=0;cnt--)
      {
         if(OrderSelect(cnt,SELECT_BY_POS))
         {
            if(OrderSymbol()==Symbol() && OrderMagicNumber() == EA_MAGIC_NUM)
            {
               positionType = OrderType();
               if (positionType == OP_BUYLIMIT)
               {
                  if (!OrderDelete(OrderTicket()))
                  {
                     OrderError();
                  }
               }
            }
         }
      }
   }
}

bool CloseAllTrades()
{
   int total = OrdersTotal()-1;
   int positionType;
   bool CloseAllSuccessful = true;
   if (total > 0)
   {
      for(int cnt=total;cnt>=0;cnt--)
      {
         if(OrderSelect(cnt,SELECT_BY_POS))
         {
            if(OrderSymbol()==Symbol() && OrderMagicNumber() == EA_MAGIC_NUM)
            {
               positionType = OrderType();
               if (positionType == OP_BUY)
               {
                  if (!OrderClose(OrderTicket(),OrderLots(),MarketInfo(OrderSymbol(),MODE_BID),3))
                  {
                     OrderError();
                     CloseAllSuccessful = false;
                  }
               }
               if (positionType == OP_SELL)
               {
                  if (!OrderClose(OrderTicket(),OrderLots(),MarketInfo(OrderSymbol(),MODE_ASK),3))
                  {
                     OrderError();
                     CloseAllSuccessful = false;
                  }
               }
               if (positionType != OP_SELL && positionType != OP_BUY) // pending orders
               {
                  if (!OrderDelete(OrderTicket()))
                  {
                     OrderError();
                     CloseAllSuccessful = false;
                  }
               }
            }
         }
      }
   }
   return (CloseAllSuccessful);
}

void DeleteOldestOfPending()
{
   int oldestTicket = 0;
   datetime oldestTicketTime = TimeCurrent();
   int pendingCount = 0;
   int total = OrdersTotal()-1;
   int positionType;
   if (total > 0)
   {
      for(int cnt=total;cnt>=0;cnt--)
      {
         if(OrderSelect(cnt,SELECT_BY_POS))
         {
            if(OrderSymbol()==Symbol() && OrderMagicNumber() == EA_MAGIC_NUM)
            {
               positionType = OrderType();
               if (positionType != OP_BUY && positionType != OP_SELL)
               {
                  pendingCount = pendingCount + 1;
                  if (OrderOpenTime() < oldestTicketTime)
                  {
                     oldestTicketTime = OrderOpenTime();
                     oldestTicket = OrderTicket();
                  }
               }
            }
         }
      }
      if (oldestTicket > 0 && pendingCount > MaxPendingOrders)
      {
         if (!OrderDelete(oldestTicket))
         {
            OrderError();
         }
      }
   }
}

void OrderError() {
  int iError=GetLastError();
  Print("Order:",OrderTicket()," GetLastError()=",iError," ",ErrorDescription(iError));
}

void WriteToLogFile(string input)
{
   string filename = "Swef_EA-"+Symbol()+"-"+Day()+"-"+Month()+"-"+Year()+".log";
   int handle = FileOpen(filename,FILE_READ|FILE_WRITE);
   if (handle>1)
   {
      FileSeek(handle, 0, SEEK_END); // go to end of file
      FileWrite(handle, input);
      FileClose(handle);
   }
}

//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
{
//----
   WriteComment();
   int CurrentHour = TimeHour(TimeCurrent());
   if (CurrentHour == OpeningHour)
   {
      double PriceToOpen;
      double TakeProfitPrice;
      double StopLossPrice;
      
      if (MACDSaysBuy() && DecideToOpenTrade(OP_BUY, OP_BUYLIMIT))
      {
         PriceToOpen = GetBuyPrice();
         if (TakeProfit == 0)
         {
            TakeProfitPrice = 0;
         }
         else
         {
            TakeProfitPrice = PriceToOpen + (TakeProfit * Point);
            TakeProfitPrice = NormalizeDouble(TakeProfitPrice,Digits);
         }
         if (StopLoss == 0)
         {
            StopLossPrice = 0;
         }
         else
         {
            StopLossPrice = PriceToOpen - (StopLoss * Point);
            StopLossPrice = NormalizeDouble(StopLossPrice,Digits);
         }
         if (Ask > PriceToOpen)
         {
            SendOrders(OP_BUYLIMIT, PositionSizeToOpen(), PriceToOpen, Slippage, StopLossPrice, TakeProfitPrice, "Swef_Buy_"+EA_MAGIC_NUM, TimeCurrent() + D'1970.01.01 01:00:00' * 24 * PendingOrderExpirationDays);
         }
         else
         {
            SendOrders(OP_BUY, PositionSizeToOpen(), Ask, Slippage, StopLossPrice, TakeProfitPrice, "Swef_Buy_"+EA_MAGIC_NUM, 0);
         }
      }
      if (MACDSaysSell() && DecideToOpenTrade(OP_SELL, OP_SELLLIMIT))
      {
         PriceToOpen = GetSellPrice();
         if (TakeProfit == 0)
         {
            TakeProfitPrice = 0;
         }
         else
         {
            TakeProfitPrice = PriceToOpen - (TakeProfit * Point);
            TakeProfitPrice = NormalizeDouble(TakeProfitPrice,Digits);
         }
         if (StopLoss == 0)
         {
            StopLossPrice = 0;
         }
         else
         {
            StopLossPrice = PriceToOpen + (StopLoss * Point);
            StopLossPrice = NormalizeDouble(StopLossPrice,Digits);
         }
         if (Bid < PriceToOpen)
         {
            SendOrders(OP_SELLLIMIT, PositionSizeToOpen(), PriceToOpen, Slippage, StopLossPrice, TakeProfitPrice, "Swef_Sell_"+EA_MAGIC_NUM, TimeCurrent() + D'1970.01.01 01:00:00' * 24 * PendingOrderExpirationDays);
         }
         else
         {
            SendOrders(OP_SELL, PositionSizeToOpen(), Bid, Slippage, StopLossPrice, TakeProfitPrice, "Swef_Sell_"+EA_MAGIC_NUM, 0);
         }
      }
   }
   if (CurrentHour == OrderCheckingHour)
   {
      if (PositionsAreInProfit())
      {
         CloseAllTrades();
      }
   }
   
   // run this checks as long as the EA is running.
   if (GetDailyMACD(0) > 0)
   {
      CloseAllPendingSellTrades();
   }
   if (GetDailyMACD(0) < 0)
   {
      CloseAllPendingBuyTrades();
   }
   if (EquityProfitOutReached())
   {
      CloseAllTrades();
   }
   if (UseEquityLossOut && EquityLossOutReached())
   {
      CloseAllTrades();
   }
   DeleteOldestOfPending();
   
//----
   return(0);
}
//+------------------------------------------------------------------+